﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Algorithms.CMAEvolutionStrategy;
using HeuristicLab.Algorithms.LocalSearch;
using HeuristicLab.Algorithms.ParameterlessPopulationPyramid;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Networks.IntegratedOptimization.TravelingThief {
  [Item("TtpNetwork3", "Version 3 of a TTP optimization network.")]
  [Creatable("Optimization Networks")]
  [StorableClass]
  public sealed class TtpNetwork3 : TtpNetwork, IOptimizer {
    [StorableConstructor]
    private TtpNetwork3(bool deserializing) : base(deserializing) { }
    private TtpNetwork3(TtpNetwork3 original, Cloner cloner) : base(original, cloner) { }
    public TtpNetwork3() : this("TtpNetwork3") { }
    public TtpNetwork3(string name) : base(name) {
      var cmaes = new CMAEvolutionStrategy();
      cmaes.Problem = new VariegationProblem();
      cmaes.MaximumGenerations = 80;
      MetaSolver.Algorithm = cmaes;
      Orchestrator.MetaSolverOrchestrationPort.ConnectedPort = MetaSolver.OrchestrationPort;

      var ls = new LocalSearch();
      ls.Problem = Orchestrator.KspParameter.Value;
      ls.MaximumIterations.Value = 100;
      ls.SampleSize.Value = 2000;
      TspSolver.Algorithm = ls;
      Orchestrator.TspSolverOrchestrationPort.ConnectedPort = TspSolver.OrchestrationPort;

      var p3 = new ParameterlessPopulationPyramid();
      p3.Problem = Orchestrator.KspParameter.Value;
      p3.MaximumRuntime = 3;
      KspSolver.Algorithm = p3;
      Orchestrator.KspSolverOrchestrationPort.ConnectedPort = KspSolver.OrchestrationPort;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TtpNetwork3(this, cloner);
    }
  }
}
